###################################################################################################
# _metadictmanager.py
#
# Implementation of class MetadictManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import copy
import string
import urllib
# Product Imports.
import _globals
import _xmllib


# -------------------------------------------------------------------------------------------------
#  getAttrName
# -------------------------------------------------------------------------------------------------
def getAttrName(id):
  s_attr = id
  s_attr = string.lower(s_attr)
  s_attr = string.replace(s_attr,'.','_')
  s_attr = 'attr_%s'%s_attr
  return s_attr

# -------------------------------------------------------------------------------------------------
#  importMetaDict
# -------------------------------------------------------------------------------------------------

def _importMetaDict(self, item):
  attr = None
  newId = item['id']
  newName = item['name']
  newType = item['type']
  newMandatory = item.get('mandatory',0)
  newMultilang = item.get('multilang',1)
  newKeys = item.get('keys',[])
  newDstMetaTypes = item.get('dst_meta_types',[])
  self.__set_metadict__(attr,newId,newName,newType,newMandatory,newMultilang,newKeys,newDstMetaTypes)

def importMetaDict(self, xml):
  builder = _xmllib.XmlAttrBuilder()
  value = builder.parse(xml)
  if type(value) is type([]):
    for item in value:
      _importMetaDict(self,item)
  else:
    _importMetaDict(self,value)
  self.synchronizeObjAttrs()


# -------------------------------------------------------------------------------------------------
#  changeMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def changeMetaDictAttrs(self, old_attr, new_attr):
  if self.meta_type[:3] == 'ZMS':
    for s_preview in [None,'preview']:
      req = {'preview':s_preview}
      ob = self.getObjVersion(req)
      for s_lang in self.getLanguages():
        old_lang_attr = "%s_%s"%(old_attr,s_lang)
        new_lang_attr = "%s_%s"%(new_attr,s_lang)
        if hasattr(ob,old_lang_attr):
          try:
            setattr(ob,new_lang_attr,copy.deepcopy(getattr(ob,old_lang_attr)))
          except:
            pass
    for ob in self.objectValues():
      changeMetaDictAttrs(ob,old_attr,new_attr)

# -------------------------------------------------------------------------------------------------
#  deleteMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def deleteMetaDictAttrs(self, attr):
  if self.meta_type[:3] == 'ZMS':
    for s_lang in self.getLanguages():
      lang_attr = "%s_%s"%(attr,s_lang)
      if hasattr(self,lang_attr):
        try:
          delattr(self,lang_attr)
        except:
          pass
    for ob in self.objectValues():
      deleteMetaDictAttrs(ob,attr)


###################################################################################################
###################################################################################################
###
###   class MetadictManager
###
###################################################################################################
###################################################################################################
class MetadictManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )
                

    # Management Interface.
    # ---------------------
    manage_customizeMetadictForm = HTMLFile('dtml/metadict/manage_customizeform', globals()) 


    # ------------------------------------------------------------------------------------------
    # MetadictManager.recurse_updateVersionMetaDicts: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionMetaDicts(self, REQUEST):
      #++ if _globals.debug: print "[%s.recurse_updateVersionMetaDicts]"%self.meta_type
      message = ""
      
      # -----------------------------------------------------------
      # 19.12.2002
      # Build #109: Allow non-multilang attributes.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '109':
        obs = []
        old_obs = copy.deepcopy(self.__get_metadicts__())
        for i in range(0,len(old_obs)/2):
          key = old_obs[i*2]
          value = old_obs[i*2+1]
          if value['type'] == 'textarea':
            value['type'] = 'text'
          if value['type'] == 'input':
            value['type'] = 'string'
          value['mandatory'] = value.get('mandatory',0)
          value['multilang'] = value.get('multilang',1)
          obs.append(key)
          obs.append(value)
        self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))

      return message


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__get_metadicts__:
    # 
    # Returns list of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def __get_metadicts__(self):
      return self.getConfProperty('ZMS.custom.metas',[])


    # ------------------------------------------------------------------------------------------
    # MetadictManager.getMetadictAttrs:
    # 
    # Returns list of attributes of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def getMetadictAttrs(self, meta_type=None):
      attrs = []
      obs = self.__get_metadicts__()
      for i in range(0,len(obs)/2):
        key = obs[i*2]
        value = obs[i*2+1]
        if meta_type is None or meta_type in self.getMetadictAttr(key)['dst_meta_types']:
          attrs.append(key)
      # ZMSLinkElement must always have attr_dc_description!
      if meta_type == 'ZMSLinkElement':
        if not 'attr_dc_description' in attrs:
          attrs.insert(0,'attr_dc_description')
      # Return attributes.
      return attrs


    # ------------------------------------------------------------------------------------------
    # MetadictManager.getMetadictAttr:
    # 
    # Get Attribute for Meta-Dictionary specified by key.
    # ------------------------------------------------------------------------------------------
    def getMetadictAttr(self, key):
      obs = self.__get_metadicts__()
      if key in obs:
        ob = obs[obs.index(key) + 1]
      # ZMSLinkElement must always have attr_dc_description!
      elif key == 'attr_dc_description':
        manage_lang = self.getManageLanguage(self.getPrimaryLanguage())
        ob = {}
        ob['id'] = 'DC.Description'
        ob['name'] = self.getLangStr('ATTR_DC_DESCRIPTION',manage_lang)
        ob['type'] = 'text'
        ob['dst_meta_types'] = [self.meta_type]
      ob = ob.copy()
      ob['key'] = getAttrName(ob['id'])
      ob['mandatory'] = ob.get('mandatory',0)
      ob['multilang'] = ob.get('multilang',1)
      ob['keys'] = ob.get('keys',[])
      ob['dst_meta_types'] = ob.get('dst_meta_types',['ZMS','ZMSRubrik','ZMSDocument'])
      ob['size'] = ob.get('size',35)
      return ob


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__set_metadict__:
    # 
    # Set/add DC.Metadictionary with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_metadict__(self, attr, id, newName, newType, newMandatory=0, newMultilang=1, newKeys=[], newDstMetaTypes=[]):
      # Remove exisiting entry.
      obs = self.__get_metadicts__()
      if attr is not None:
        i = obs.index(attr)
        del obs[i] 
        del obs[i] 
      else: 
        i = len(obs)
      # Attribute.
      attr = getAttrName(id)
      # Values.
      values = {}
      values['id'] = id
      values['name'] = newName
      values['type'] = newType
      values['mandatory'] = newMandatory
      values['multilang'] = newMultilang
      values['keys'] = newKeys
      values['dst_meta_types'] = newDstMetaTypes
      # Update attribute.
      obs.insert(i,values)
      obs.insert(i,attr)
      self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))


    # ------------------------------------------------------------------------------------------
    #	MetadictManager.__move_metadict__:
    #
    #	Moves DC.Metadictionary with specified ATTR in specified DIRECTION.
    # ------------------------------------------------------------------------------------------
    def __move_metadict__(self, attr, dir):
      # Move.
      obs = self.__get_metadicts__()
      i = obs.index(attr)
      attr = obs[i] # Attribute
      values = obs[i+1] # Values
      del obs[i] 
      del obs[i] 
      # Update attribute.
      obs.insert(i+dir*2,values)
      obs.insert(i+dir*2,attr)
      self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__del_metadict__:
    # 
    # Delete DC.Metadictionary with specified ATTR.
    # ------------------------------------------------------------------------------------------
    def __del_metadict__(self, attr):
      # Delete.
      obs = self.__get_metadicts__()
      i = obs.index(attr)
      # Update attribute.
      del obs[i] # Attribute
      del obs[i] # Values
      self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))


    ###############################################################################################
    # MetadictManager.manage_changeMetadicts:
    #
    # Werte fr DC.Metadictionaries lschen oder ndern.
    ###############################################################################################
    def manage_changeMetadicts(self, btn, lang, manage_lang, REQUEST, RESPONSE=None):
        """ MetadictManager.manage_changeMetadicts """        

        message = ''
        attr = REQUEST.get('attr','')
        
        
        # Delete.
        # -------
        if btn in ['delete',self.getLangStr('BTN_DELETE',manage_lang)]:
          self.__del_metadict__(attr)
          deleteMetaDictAttrs(self,attr)
          message = self.getLangStr('MSG_CHANGED',manage_lang)
          attr = ''
                    
        # Change.
        # -------
        elif btn == self.getLangStr('BTN_CHANGE',manage_lang): 
          # Change.
          id = string.strip(REQUEST['id'])
          newName = string.strip(REQUEST['name'])
          newType = string.strip(REQUEST['metatype'])
          newMandatory = REQUEST.get('mandatory',0)
          newMultilang = REQUEST.get('multilang',0)
          newKeys = self.string_list(REQUEST['keys'],'\n')
          newDstMetaTypes = REQUEST.get('dst_meta_types',[])
          self.__set_metadict__(attr,id,newName,newType,newMandatory,newMultilang,newKeys,newDstMetaTypes)
          #
          new_attr = getAttrName(id)
          if attr != new_attr:
            changeMetaDictAttrs(self,attr,new_attr)
            deleteMetaDictAttrs(self,attr)
          # Display attribute..
          attr = new_attr
          message = self.getLangStr('MSG_CHANGED',manage_lang)
        
        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          value = []
          attrs = REQUEST.get('attrs',[])
          metadicts = self.__get_metadicts__()
          for i in range(0,len(metadicts)/2):            
            id = metadicts[i*2]
            dict = metadicts[i*2+1]
            if id in attrs or len(attrs) == 0:
              value.append(dict)
          if len(value)==1:
            value = value[0]
          content_type = 'text/xml'
          filename = 'ZMS.custom.metas.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + _xmllib.toXml(self,value,1)
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export

        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          importMetaDict(self,xml=REQUEST['file'])
          message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%REQUEST['file'].filename)

        # Move up.
        # --------
        elif btn == 'move_up':
          self.__move_metadict__(attr,-1)
          message = self.getLangStr('MSG_MOVEDOBJUP',manage_lang)%("<i>%s</i>"%attr)
          attr = ''
        
        # Move down.
        # ----------
        elif btn == 'move_down':
          self.__move_metadict__(attr,+1)
          message = self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%attr)
          attr = ''
        
        # Insert.
        # -------
        elif btn == self.getLangStr('BTN_INSERT',manage_lang): 
          newId = string.strip(REQUEST['_id'])
          newName = string.strip(REQUEST['_name'])
          newType = string.strip(REQUEST['_type'])
          newMandatory = REQUEST.get('_mandatory',0)
          newMultilang = REQUEST.get('_multilang',1)
          self.__set_metadict__(None,newId,newName,newType,newMandatory,newMultilang)
          # Display attribute..
          attr = getAttrName(newId)
          
        ##### SYNCHRONIZE ####
        self.synchronizeObjAttrs()
        
        # Return with message.
        message = urllib.quote(message)
        return REQUEST.RESPONSE.redirect('manage_customizeMetadictForm?attr=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(attr,lang,manage_lang,message))

###################################################################################################
