###################################################################################################
# _scormlib.py
#
# Implementation of class SCORMLib (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################


# Product Imports.
import zmscontainerobject
import _xmllib


# ---------------------------------------------------------------------------------------------
#  getIMSManifestItems:
# ---------------------------------------------------------------------------------------------
def getIMSManifestItems(self, REQUEST):
  obs = self.filteredChildNodes(REQUEST)
  xml = ''
  xml = xml + '<item'
  xml = xml + ' identifier="%s"'%self.id
  if self.meta_type=='ZMSCustom' or zmscontainerobject.isPageWithElements(obs):
    xml = xml + ' identifierref="R_%s"'%self.id
  xml = xml + '>'
  xml = xml + '<title>%s</title>\n'%_xmllib.xml_quote(self.getTitlealt(REQUEST))
  for ob in obs:
    if ob.isPage():
      xml = xml + getIMSManifestItems(ob,REQUEST)
  xml = xml + '</item>'
  return xml

# ---------------------------------------------------------------------------------------------
#  getIMSManifestResources:
# ---------------------------------------------------------------------------------------------
def getIMSManifestResources(base, self, REQUEST):
  obs = self.filteredChildNodes(REQUEST)
  xml = ''
  if self.meta_type=='ZMSCustom' or zmscontainerobject.isPageWithElements(obs):
    href =  self.getHref2IndexHtml(REQUEST)[len(base.absolute_url()):]
    xml = xml + '<resource'
    xml = xml + ' identifier="R_%s"'%self.id
    xml = xml + ' type="webcontent"'
    xml = xml + ' adlcp:scormtype="sco"'
    xml = xml + ' href="%s"'%href
    xml = xml + '>\n'
    xml = xml + '<metadata>\n'
    xml = xml + '<schema>ADL SCORM</schema>\n'
    xml = xml + '<schemaversion>1.2</schemaversion>\n'
    xml = xml + '</metadata>\n'
    xml = xml + '<file href="%s" />'%href
    xml = xml + '</resource>\n'
  for ob in obs:
    if ob.isPage():
      xml = xml + getIMSManifestResources(base,ob,REQUEST)
  return xml


###################################################################################################
###################################################################################################
###
###   class SCORMLib
###
###################################################################################################
###################################################################################################
class SCORMLib:

  # ---------------------------------------------------------------------------------------------
  #  parseIMSManifest:
  #
  #  Parse IMSManifest.xml
  # ---------------------------------------------------------------------------------------------
  def parseIMSManifest(self, xml):
    IMSManifest = []
    NodeSet = self.xmlParse(xml)
    ResourceDict = {}
    for nResources in self.xmlNodeSet(NodeSet,'resources'):
      for nResource in self.xmlNodeSet(nResources,'resource'):
        attrs = nResource['attrs']
        identifier = attrs['identifier']
        ResourceDict[identifier] = attrs
    for nOrganizations in self.xmlNodeSet(NodeSet,'organizations'):
      for nOrganization in self.xmlNodeSet(nOrganizations,'organization'):
        level_offs = nOrganization['level']
        for nItem in self.xmlNodeSet(nOrganization,'item',1):
          attrs = nItem['attrs']
          level = nItem['level']
          identifier = attrs['identifier']
          identifierref = attrs.get('identifierref',None)
          item = {}
          item['id'] = identifier
          item['level'] = level-level_offs
          item['title'] = None
          for nTitle in self.xmlNodeSet(nItem,'title'):
            item['title'] = nTitle['cdata']
          item['href'] = None
          if identifierref in ResourceDict.keys():
            item['href'] = ResourceDict[identifierref]['href']
          IMSManifest.append(item)
    return IMSManifest

  # ---------------------------------------------------------------------------------------------
  #  getIMSManifest:
  #
  #  Returns IMSManifest.xml
  # ---------------------------------------------------------------------------------------------
  def getIMSManifest(self, REQUEST):
    xml = ''
    xml = xml + '<?xml version="1.0" encoding="ISO-8859-1"?>\n'
    xml = xml + '<manifest identifier="%sManifest" version="1.1"\n'%self.id
    xml = xml + 'xmlns="http://www.imsproject.org/xsd/imscp_rootv1p1p2"\n'
    xml = xml + 'xmlns:adlcp="http://www.adlnet.org/xsd/adlcp_rootv1p2"\n'
    xml = xml + 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n'
    xml = xml + 'xsi:schemaLocation="http://www.imsproject.org/xsd/imscp_rootv1p1p2 imscp_rootv1p1p2.xsd\n'
    xml = xml + 'http://www.imsglobal.org/xsd/imsmd_rootv1p2p1 imsmd_rootv1p2p1.xsd\n'
    xml = xml + 'http://www.adlnet.org/xsd/adlcp_rootv1p2 adlcp_rootv1p2.xsd">\n'
    xml = xml + '<metadata/>\n'
    xml = xml + '<organizations default="%s">\n'%self.id
    xml = xml + '<organization default="%s">\n'%self.id
    xml = xml + '<title>%s</title>\n'%_xmllib.xml_quote(self.getTitlealt(REQUEST))
    xml = xml + getIMSManifestItems(self,REQUEST)
    xml = xml + '</organization>\n'
    xml = xml + '</organizations>\n'
    xml = xml + '<resources>\n'
    xml = xml + getIMSManifestResources(self,self,REQUEST)
    xml = xml + '</resources>\n'
    xml = xml + '</manifest>\n'
    return xml;
  
  
###################################################################################################
